from sphinx.application import Sphinx

from sphinx_notion.builders import NotionBuilder

__version__ = "0.0.5"

# https://developers.notion.com/reference/request-limits#size-limits
NOTION_API_RICH_TEXT_CONTENT_CHARACTER_LIMIT = 2_000


def setup(app: Sphinx):
    app.add_config_value(
        "sphinx_notion_code_block_character_limit",
        NOTION_API_RICH_TEXT_CONTENT_CHARACTER_LIMIT,
        "env",
        description=(
            "Character limit for Sphinx code-blocks. "
            "Cannot exceed 2000 characters, "
            "as it is the size limit of Notion's rich_text.content."
        ),
    )
    app.add_builder(NotionBuilder)



from pygments.lexers import get_lexer_by_name

PygmentsLanguage = str


def get_standard_pygments_language(language: str) -> PygmentsLanguage:
    # "default" language means "not specified"
    if language == "default":
        return "default"
    lexer = get_lexer_by_name(language)
    # Lexer has aliases but mypy raises this error:
    # >error: "Lexer" has no attribute "aliases"  [attr-defined]
    # https://github.com/pygments/pygments/blob/2.19.2/pygments/lexer.py#L111-L113
    return lexer.aliases[0]  # type: ignore[attr-defined]


def to_notion_language(pygments_language: PygmentsLanguage) -> str:
    if pygments_language in {"default", "pytb", "text", "output"}:
        return "plain text"
    if pygments_language in {"python", "pycon"}:
        return "python"
    # TODO: Support for other languages
    return pygments_language


def chunk_code(code: str, upper_limit: int):
    lines = code.splitlines(keepends=True)
    max_line_length = max(len(line) for line in lines)
    if max_line_length > upper_limit:
        raise ValueError(
            f"upper_limit (current {upper_limit}) needs to be greater "
            f"than max_line_length ({max_line_length})"
        )

    buffer = ""
    for line in lines:
        if len(buffer) + len(line) <= upper_limit:
            buffer += line
        else:
            yield buffer
            buffer = line
    if buffer:
        yield buffer
